/**
 *   URL:
 *   说明:
 *   负责人: kelen
 *   日期:  8/3 0003.
 *
 *
 *   var options = [{ value: 1, text: '男' }, {value: 0, text: '女'}]
     $(".searchForm").searchForm({
        action: "/admin/admin-car-library",
        method: "post",
        inputGroup: [
            { label: '姓名', name: 'name', type: 'text'},
            { label: '年龄', name: 'age', type: 'text'},
            { label: '类型', name: 'type', type: 'select', options: options }
            { label: '预约开始时间', name: 'orderStartTime', type: 'time' },
        ],
        searchCar: true
     });
 *
 */
(function($) {
    $.fn.searchForm = function(options) {

        var defaultOptions = {
            action: "",             // 必选 提交URL
            searchCar: false,       // 默认不显示汽车查询
            method: 'get',          // 默认’get‘方法
            inputGroup: [],          // input组,
            defaultInputType: "text",
            success: $.noop,
            error: $.noop,
            complete: $.noop,
            pagination: {
            	totalPageKey: "", // 返回对象所有页面的json键
            	nowPageKey: "",		// 返回对象当前页面的json键
                pageSizeKey: "",
            	selector: '',  //  显示分页的元素
            }
        };

        var opts = $.extend(defaultOptions, options);

        var instance = this;        // 实例对象
        instance.id = Math.random();
        var form = document.createElement("form");      // 表单
        form.id = "searchForm" + instance.id;
        // form.method = opts.method;
        // form.action = opts.action;
        form.className = "form-inline cio-search-form";


        if (!opts.action) {
            throw new Error('表单action必须设置');
        }

        initForm();
        initEvent();

        function initForm() {
            // 文本输入框
            instance.append(generateForm(opts.inputGroup));

            if (!$.fn.appendDtpicker) {
                throw new Error('must include simple-dtpicker plugin')
            }
            $(".time-selector").appendDtpicker({
                locale: 'cn',
                minuteInterval: 15,
                autodateOnStart: false,
            });

            setTimeout(function() {
                sendForm($("#searchForm").serialize());
            }, 80)

        }

        function initEvent() {
            bindSubmitEvent();      // 绑定提交事件
            bindPaginationEvent();  // 绑定分页按钮点击事件
        }

        // 生成表单
        function generateForm(inputGroup) {
            // <div class="form-group">
            //     <label for="">标题</label>
            //     <input class="form-control" name="title" type="text" value="">
            // </div>
            for (var i = 0; i < inputGroup.length; i++) {
                var inputItem = inputGroup[i];
                var div = document.createElement("div");
                div.className = "form-group";
                var label = document.createElement("label");
                label.innerText = inputItem['label'];
                div.appendChild(label);
                div.appendChild(generateInput(inputItem));
                form.appendChild(div);
            }
            // 生成搜索按钮
            var searchBtn = document.createElement("button");
            searchBtn.type = "button";
            searchBtn.id = "searchFormSubmitBtn" + instance.id
            searchBtn.innerText = "搜索";
            searchBtn.className = "btn btn-primary";
            form.appendChild(searchBtn);
            return form;
        }

       
        // 注册提交事件
        function bindSubmitEvent() {
            document.getElementById("searchFormSubmitBtn" + instance.id).onclick = function() {
                sendForm($("#searchForm" + instance.id).serialize());
            }
        }

        // 表单提交
        function sendForm(data) {
            $.ajax({
                url: opts.action,
                type: opts.method,
                data: data,
                processData: false,
                success: function (data) {
                    opts.success.call(instance, data);
                    if (opts.pagination) {
                        // console.log(loadPageAjax(data[opts.pagination.nowPageKey], data[opts.pagination.pageSizeKey]));
                        $(opts.pagination.selector).html(loadPageAjax(data[opts.pagination.nowPageKey], data[opts.pagination.pageSizeKey]));  // 添加分页
                    }
                },
                error: function(data) {
                    opts.error(data);
                },
                complete: function(data) {
                    opts.complete(data);
                }
            })
        }

        // 加载ajax分页
        function loadPageAjax (nowPage,pageCount){
            var nowPage = parseInt(nowPage);
            var pageCount = parseInt(pageCount);
            var pageHtml = '';
            var start, end, pageOffset = 5;
            pageHtml += '<li data-page="'+ 1 +'"><span aria-hidden="true">&laquo;&laquo;</span></li>';
            if (nowPage > 1) {
                pageHtml += '<li data-page="'+ (nowPage - 1) +'"><span aria-hidden="true">&laquo;</span></li>';
            } else {
                pageHtml += '<li class="disabled"><span aria-hidden="true">&laquo;</span></li>';
            }
            if (pageCount <= pageOffset) {
                start = 1;
                end = pageCount;
            } else {
                if (nowPage < pageOffset) {
                    start = 1;
                    end = pageOffset;
                } else if (nowPage > pageCount - 2) {
                    start = pageCount - 4;
                    end = pageCount;
                } else {
                    start = nowPage - 2;
                    end = nowPage + 2;
                }
            }
            for (var i = start; i <= end; ++i) {
                pageHtml += '<li ' + (i == nowPage ? 'class="active"': "") + ' data-page="'+ i +'"><span aria-hidden="true">' + i + '</span></li>';
            }
            if (nowPage < pageCount) {
                pageHtml += '<li data-page="'+ (nowPage + 1) +'"><span aria-hidden="true">&raquo;</span></li>';
            } else {
                pageHtml += '<li class="disabled"><span aria-hidden="true">&raquo;</span></li>';
            }
            pageHtml += '<li data-page="'+ pageCount +'"><span aria-hidden="true">&raquo;&raquo;</span></li>';
            pageHtml += '<li><span>' + nowPage + '/' + pageCount + '</span></li>'
            return pageHtml;
        }

        // 分页按钮点击事件
        function bindPaginationEvent() {
            if (opts.pagination) {
                $(opts.pagination.selector).on('click', 'li', function() {
                    // 分页按钮点击事件
                    var nowPage = $(this).attr('data-page');
                    if (nowPage) {
                        var params = $("#searchForm" + instance.id).serialize();
                        params += "&nowPage=" + nowPage;
                        sendForm(params)
                    }
                })
            }
        }
        // 打印控制台
        function log(msg) {
            console.log(msg);
        }

        // 生成input
        function generateInput (inputItem) {
            var inputElem = null;
            switch (inputItem.type) {
                case 'select': {
                    inputElem = document.createElement("select");
                    inputElem.className = "form-control";
                    inputElem.name = inputItem['name'];
                    for (var i = 0; i < inputItem.options.length; i++) {
                        var option = document.createElement('option');
                        option.value = inputItem.options[i].value;
                        option.innerText = inputItem.options[i].text;
                        if (inputItem.options[i].value == inputItem['default'])
                            option.selected = true;
                        inputElem.appendChild(option);
                    }
                    break;
                }
                case 'time': {
                    if (inputItem.range) {
                        // 时间范围
                        inputElem = document.createElement("span");
                        for (var i = 0; i < 2; i++) {
                            var input = document.createElement("input");
                            input.className = "form-control time-selector";
                            input.type = "text";
                            input.name = inputItem['name'];
                            inputElem.appendChild(input);
                        }
                    } else {
                        inputElem = document.createElement("input");
                        inputElem.className = "form-control time-selector";
                        inputElem.type = "text";
                        inputElem.name = inputItem['name'];
                    }
                    break;
                }
                default: {
                    inputElem = document.createElement("input");
                    inputElem.className = "form-control";
                    inputElem.type = inputItem['type'] || opts.defaultInputType;
                    inputElem.name = inputItem['name'];
                    inputElem.value = inputItem['default'] || '';
                }
            }
            return inputElem;
        }

        this.refresh = function(sendData) {
            var sendData = sendData || {};
            sendForm(sendData);
        }

        // 把字符串转为数字
        this.resolveInt = function (num) {
            return parseInt(num) ? parseInt(num): 0;
        }
        // 转为浮点
        this.resolveFloat = function (num) {
            return parseFloat(num) ? parseFloat(num).toFixed(2): 0.00;
        }
        this.resolveString = function (str) {
            return typeof str == 'object' ? JSON.stringify(str) : str.toString();
        }
        this.resolveTime = function (timeStr, fmt) {
            var res = '';
            fmt = fmt || "yyyy-MM-dd hh:mm:ss";
            if (new Date(timeStr) != 'Invalid Date') {
                var time = new Date(timeStr);
                var o = {
                    "M+": time.getMonth() + 1, //月份
                    "d+": time.getDate(), //日
                    "h+": time.getHours(), //小时
                    "m+": time.getMinutes(), //分
                    "s+": time.getSeconds(), //resolveInt
                    "q+": Math.floor((time.getMonth() + 3) / 3), //季度
                    "S": time.getMilliseconds() //毫秒
                };
                if (/(y+)/.test(fmt)) fmt = fmt.replace(RegExp.$1, (time.getFullYear() + "").substr(4 - RegExp.$1.length));
                for (var k in o)
                    if (new RegExp("(" + k + ")").test(fmt)) fmt = fmt.replace(RegExp.$1, (RegExp.$1.length == 1) ? (o[k]) : (("00" + o[k]).substr(("" + o[k]).length)));
                res = fmt;
            }
            return res;
        }
        return this;
    }
})(jQuery)